#!/usr/bin/env python

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Devin Howard

import pyaudio, wave
import gtk, pygtk, gobject
pygtk.require('2.0')
gtk.gdk.threads_init ()

import sys
from threading import Thread
from time import sleep

from gui import State, GUI, WritingWindow

#global constants
FORMAT = pyaudio.paInt16
CHANNELS = 1
CHUNK = 1024
RATE = 44100
snapLenDefault = 180
filenameDefault = "record.wav"

class Recorder (Thread):
    def __init__ (self, snapLen):
        self.task = State.recording
        self.filename = filenameDefault
        self.snapshotLength = snapLen
        Thread.__init__(self)

    def snap (self):
        self.task = State.writing

    def recordingTask (self, stream, all, rc, i):
        data = stream.read (CHUNK)
        all [i] = data
        self.task = State.recording

    def writingTask (self, p, all, i):
        writingWindow = WritingWindow ()
        snap1 = all [i:]
        snap2 = all [:i]
        data = []
        for chunk in snap1:
            data += chunk
        for chunk in snap2:
            data += chunk
        wf = wave.open(self.filename, 'wb')
        wf.setnchannels(CHANNELS)
        wf.setsampwidth(p.get_sample_size(FORMAT))
        wf.setframerate(RATE)
        wf.writeframes(str(data))
        wf.close()

        gobject.idle_add(self.closeWW, writingWindow)

    def closeWW (self, ww):
        sleep (0)
        ww.destroy ()

    def run (self):
        p = pyaudio.PyAudio()

        stream = p.open(format = FORMAT,
                        channels = CHANNELS,
                        rate = RATE,
                        input = True,
                        frames_per_buffer = CHUNK)
        self.all = [str () for i in range (RATE / CHUNK * self.snapshotLength)]
        i = 0
        rc = RATE / CHUNK

        while True:
            if self.task == State.recording:
                self.task = State.waiting
                gobject.idle_add(self.recordingTask, stream, self.all, rc, i)
                i = (i + 1) % (rc * self.snapshotLength)
            elif self.task == State.waiting:
                sleep (0)
            elif self.task == State.writing:
                gobject.idle_add(self.writingTask, p, self.all, i)
                self.task = State.recording
            elif self.task == State.quitting:
                break
        stream.close()
        p.terminate()
#End Recorder class

def main():
    #RunLoop's init declares a Recorder
    thread = Recorder (snapshotLength)
    thread.start () 
    GUI(thread)

    gtk.main()
    return 0       

if __name__ == "__main__":
    snapshotLength = snapLenDefault
    if len(sys.argv) == 2:
        snapshotLength = int (sys.argv [1])
    elif len (sys.argv) > 2:
        print "Usage: " + sys.argv[0] + " [snapshotLength]"
        sys.exit (1)

    main()
